#ifndef __CAMERA_H__
#define __CAMERA_H__

#include <cglm/cglm.h>
#include <cglm/cam.h>
#include <cglm/euler.h>
#include <cglm/types.h>
#include <cglm/quat.h>
#include <cglm/util.h>
struct camera{
    mat4 view;
    mat4 projection;
    vec3 position;
    vec4 rotation; // quaternion
    float fov;
    float aspectRatio;
    float far; // far clipping plane
    float near; // near clipping plane

    // up forward and right vectors
    vec3 up; 
    vec3 forward;
    vec3 right;
};

static inline void camera_update(struct camera *cam){
    glm_perspective(cam->fov, cam->aspectRatio, cam->near, cam->far, cam->projection);

    // correct the rotations to be righthanded, z-up and not upside down
    vec3 angles = {glm_rad(90.0), 0.f, 0.f};
    vec4 rotation;
    glm_euler_xyz_quat(angles, rotation);
    glm_quat_mul(cam->rotation, rotation, rotation);

    glm_quat_look(cam->position, rotation, cam->view);

    vec3 up = {0.f,0.f,-1.f};
    vec3 forward = {0.f,1.f,0.f};
    vec3 right = {1.f,0.f,0.f};

    glm_quat_rotatev(cam->rotation, up, cam->up);
    glm_quat_rotatev(cam->rotation, forward, cam->forward);
    glm_quat_rotatev(cam->rotation, right, cam->right);
}

static inline void camera_angleAxis(struct camera *cam, float angle, vec3 axis){
    vec4 quat;
    glm_quatv(quat, angle, axis);
    glm_quat_mul(cam->rotation, quat, cam->rotation);
}

#endif