#include "elementHeap.h"
#include <stdint.h>
#include <stdlib.h>
#include <sys/types.h>

#if __amd64__
//#define ASM_BSR_AVAIL
#endif

void *elementHeap_getFreeElement(struct elementHeap *h){
    uint32_t index = elementHeap_getFreeIndex(h);
    if(index != UINT32_MAX)
        return h->buffer + h->objsize * index;
    else
        return NULL;
}
uint32_t elementHeap_getFreeIndex(struct elementHeap *h){
    uint32_t avblen = h->len / 32 + (h->len % 32 != 0); // get the lenth of the availability bitmap
    for (uint32_t i = 0; i < avblen; i++) {
        uint32_t j = (i + h->lkg) % avblen; // start searching from the lkg
        if(h->availabilitybitmap[j] != UINT32_MAX){
            #ifdef ASM_BSR_AVAIL
            // potential future stuff
            #else
            
            uint32_t x = h->availabilitybitmap[j];
            uint32_t lzc = 0
                + ((x & 0xFFFF0000) ? 16 : 0)
                + ((x & 0xFF00FF00) ? 8 : 0)
                + ((x & 0xF0F0F0F0) ? 4 : 0)
                + ((x & 0xCCCCCCCC) ? 2 : 0)
                + ((x & 0xAAAAAAAA) ? 1 : 0); // get the specific bit of the unset element
            
            h->availabilitybitmap[j] |= 1 << lzc; // set the bit for the reserved element
            h->lkg = j; // update the lkg

            if(j * 32 + lzc >= h->len) return UINT32_MAX;

            return j * 32 + lzc;

            #endif
        }
    }

    // return uint32 maximum if no free UBO is found.
    return UINT32_MAX;
}

void elementHeap_init(struct elementHeap *h){
    uint32_t avblen = h->len / 32 + (h->len % 32 != 0); // get the lenth of the availability bitmap
    h->availabilitybitmap = calloc(1, avblen * sizeof(uint32_t));
}