#ifndef __UNIFORMS_H__
#define __UNIFORMS_H__

// a Bundle is data for multiple objects put into a buffer
#include <stdint.h>

struct uboBufinfo{
    struct uniformTable *ut;
    uint32_t id;
};

// a heap of elements with a bitmap for availability
struct elementHeap{
    uint32_t *availabilitybitmap; // bit field specifying which uniforms are available
    uint32_t len; // length of the heap in elements
    uint32_t objsize; // the size of one of the objects
    uint32_t lkg; // last known good index to start searching from in elements of availabilitybitmap
    void *buffer; // the data (on gpu, expect write only)
};

// set an element of the heap as reserved (does not check) (avoid using this unless a specific element is required)
static inline void elementHeap_setReserved(struct elementHeap *h, uint32_t index){
    h->availabilitybitmap[index/32] |= 1 << (index % 32);
}

// set an element of the heap as free (does not check)
static inline void elementHeap_setFree(struct elementHeap *h, uint32_t index){
    h->availabilitybitmap[index/32] &= ~(1 << (index % 32));
    h->lkg = index/32;
}

// get the index of the first free element without setting it as used (avoid using this)
uint32_t elementHeap_getFreeIndex(struct elementHeap *h);

// get the index of the first free element without setting it as used (avoid using this)
static inline uint32_t elementHeap_getFreeIndexAndReserve(struct elementHeap *h){
    uint32_t i = elementHeap_getFreeIndex(h);
    elementHeap_setReserved(h, i);
    return i;
}

// get the first free element and return a pointer to it whilst setting it as used
void *elementHeap_getFreeElement(struct elementHeap *h);

// initialise the heap, BUFFER IS NOT CREATED, _MUST_ BE CREATED SEPERATLEY
void elementHeap_init(struct elementHeap *h);

#endif