#include "data.h"
#include "setup.h"
#include "../vulkan/swapchain.h"
#include <cglm/quat.h>
#include <cglm/vec3.h>
#include <stdint.h>
#include <string.h>
#include <vulkan/vulkan_core.h>

void uniformData_init(struct uniformData *ud, struct elementHeap *heap){
    ud->heap = heap;
    ud->index = elementHeap_getFreeIndexAndReserve(heap);
}
void renderData_create(struct rendering_state *s, struct vertex *verts, uint32_t vertslen, uint32_t *indicies, uint32_t indiceslen, struct renderData *obj){
    obj->vertexBuffer.size = sizeof(struct vertex) * vertslen;
    obj->indexBuffer.size = sizeof(uint32_t) * indiceslen;

    rendering_makeBufferForRender(&obj->vertexBuffer, verts, VK_BUFFER_USAGE_VERTEX_BUFFER_BIT, VK_SHARING_MODE_EXCLUSIVE, s);
    rendering_makeBufferForRender(&obj->indexBuffer, indicies, VK_BUFFER_USAGE_INDEX_BUFFER_BIT, VK_SHARING_MODE_EXCLUSIVE, s);

    obj->indexCount = indiceslen;
    obj->uniforms.heap = &s->pipeline.uniformHeap;
    obj->uniforms.index = elementHeap_getFreeIndexAndReserve(obj->uniforms.heap);
    printf("index = %i\n", obj->uniforms.index);
    obj->uniforms.ubo = obj->uniforms.heap->buffer + obj->uniforms.heap->objsize * obj->uniforms.index;

    glm_quat_copy(GLM_QUAT_IDENTITY, obj->rotation);
    vec3 scaleident = {1.f, 1.f, 1.f};
    glm_vec3_copy(scaleident, obj->scale);
    vec3 origin = {0.f,0.f,0.f};
    glm_vec3_copy(origin, obj->position);

}

const VkSamplerCreateInfo p3ddefaultSamplerInfo = {
    VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO,
    NULL,
    0,
    VK_FILTER_LINEAR,
    VK_FILTER_LINEAR,
    VK_SAMPLER_MIPMAP_MODE_NEAREST,
    VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE,
    VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE,
    VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE,
    0.f,
    VK_FALSE,
    0.f,
    VK_FALSE,
    VK_COMPARE_OP_ALWAYS,
    0.f,
    0.f,
    VK_BORDER_COLOR_INT_TRANSPARENT_BLACK,
    VK_FALSE
};

const VkSamplerCreateInfo *p3dgetDefaultSamplerCi(){
    return &p3ddefaultSamplerInfo;
}

void renderData_setupTextures(struct rendering_state *s, struct renderData *data, VkSamplerCreateInfo *samplerOverride){
    svlk_createVkImageView(s->device, data->albedo.image, VK_IMAGE_VIEW_TYPE_2D, data->albedo.fmt, &data->albedo.view);
    vkCreateSampler(s->device, samplerOverride ? samplerOverride : p3dgetDefaultSamplerCi(), NULL, &data->albedo.sampler);

    createObjTextureDescriptorSet(s, data);
    updateObjTextureDescriptorSet(s, data);
}