#ifndef __VERTEX_H__
#define __VERTEX_H__
#include <cglm/types.h>
#include <vulkan/vulkan_core.h>
#include "../vulkan/gpumem.h"
#include "../management/elementHeap.h"

struct vertex {
    //position
    float x;
    float y;
    float z;

    // texture
    float u;
    float v;
};

struct Pipeline3dUniformBufferObject{
    mat4 matrix;
};

struct uniformData {
    struct elementHeap *heap;
    uint32_t index;
    struct Pipeline3dUniformBufferObject *ubo;
};

// data for generic meshes (not things like heightmaps or other exotic things)
struct renderData {
    struct gpubuffer vertexBuffer;
    struct gpubuffer indexBuffer;
    uint32_t indexCount;
    struct uniformData uniforms;
    vec3 position;
    vec4 rotation;
    vec3 scale;
    struct gpuimage albedo;
    VkDescriptorSet textureDescSet;
};

void uniformData_init(struct uniformData *ud, struct elementHeap *heap);

void renderData_create(struct rendering_state *s, struct vertex *verts, uint32_t vertslen, uint32_t *indicies, uint32_t indiceslen, struct renderData *obj);

void renderData_setupTextures(struct rendering_state *s, struct renderData *data, VkSamplerCreateInfo *samplerOverride);

const VkSamplerCreateInfo *getDefaultSamplerCi();

#endif