#ifndef __PIPELINE3D_PIPELINE_H__
#define __PIPELINE3D_PIPELINE_H__

#include <vulkan/vulkan_core.h>
#include "../management/elementHeap.h"
#include "../vulkan/gpumem_types.h"


/*
 * Pipeline3d.
 *  this is a 3d pipeline designed for generic 3d game stuff with shadows and lighting (todo (shadows & lighting) )
 *  triangles are indexed (with index buffer) matricies are calculated per object on the cpu
 *  depth pre pass with some defered rendering ish
 *  
 *  Textures:
 *   Albedo
 *   Normal     TODO
 *
 *  Textures (runtime):
 *   Out/Albedo
 *   Depth      TODO
 *   Normal     TODO
 *   Position   TODO
 *   Lighting   TODO
 *   
 *
 *  Pipeline:
 *   Monolithic shader renders models and optionally lighting
 *   Post processing shaders do stuff with the runtime textures                 TODO
 *   Merger merges lighting runtime texture with output (optional)              TODO
 *   Post postproccesser shaders do stuff with lit scene (optional optional)    TODO
 */

struct pipeline3dSubpass {
    VkPipelineLayout layout;
    VkPipeline pipeline;
    // next two must be in this order (used as an array, should change. too lazy)
    VkDescriptorSetLayout uniformDesclayout;
    VkDescriptorSetLayout textureDesclayout;
};

struct pipeline3d {
    VkRenderPass renderpass;
    struct pipeline3dSubpass geometryPass;
    VkDescriptorPool descriptorPool;
    // potentially wrap the uniform heap in a linked list
    struct elementHeap uniformHeap;
    VkDescriptorSet uniformDescriptorSet;
    struct gpubuffer uniformbuffer;
};

#endif