#include "render.h"
#include "pipeline.h"
#include <cglm/affine-pre.h>
#include <cglm/affine.h>
#include <cglm/mat4.h>
#include <cglm/quat.h>
#include <stdint.h>
#include <vulkan/vulkan_core.h>
#include "../camera.h"

void pipeline3d_recordCommandBuffer(VkCommandBuffer cmdbuf, VkRenderPass rp, VkImageView target, VkExtent2D targetextent, VkImageLayout targetlayout, struct rendering_state *s, struct pipeline3d *p, struct renderData *obj, uint32_t objcount, struct camera *cam){
    VkClearValue clearcolor = {0.4f,0.f,0.f,0.5f};

    VkRenderingAttachmentInfo attachments[1] = {
        {
            VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO,
            NULL,
            target,
            targetlayout,
            VK_RESOLVE_MODE_NONE,
            0,
            0,
            VK_ATTACHMENT_LOAD_OP_DONT_CARE,
            VK_ATTACHMENT_STORE_OP_STORE,
            clearcolor
        }
    };

    VkRenderingInfo renderinfo = {
        VK_STRUCTURE_TYPE_RENDERING_INFO,
        NULL,
        0,
        {{0,0},targetextent},
        1,
        0,
        1,
        attachments,
        NULL,
        NULL
    };

    vkCmdBeginRendering(cmdbuf, &renderinfo);

    vkCmdBindPipeline(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->geometryPass.pipeline);
    
    VkViewport vp = {
        0, 0,
        s->window_width, s->window_height,
        0.f, 1.f
    };
    vkCmdSetViewport(cmdbuf, 0, 1, &vp);

    VkRect2D sc = {{0,0}, {s->window_width, s->window_height}};
    vkCmdSetScissor(cmdbuf, 0, 1, &sc);

    mat4 projview;
    glm_mat4_mul(cam->projection, cam->view, projview);

    for (uint32_t i = 0; i < objcount; i++) {

        // camera stuff
        mat4 model;
        glm_mat4_dup(GLM_MAT4_IDENTITY, model);
        glm_scale(model, obj[i].scale);
        glm_translate(model, obj[i].position);
        glm_quat_rotate(model, obj[i].rotation, model);

        //glm_mat4_dup(projview, obj[i].uniforms.ubo->matrix);
        glm_mat4_mul(projview, model, obj[i].uniforms.ubo->matrix);
        

        VkDeviceSize vtxoffsets[] = {0};
        uint32_t unioffsets[] = {obj[i].uniforms.index};
        vkCmdBindVertexBuffers(cmdbuf, 0, 1, &obj[i].vertexBuffer.buffer, vtxoffsets);
        vkCmdBindIndexBuffer(cmdbuf, obj[i].indexBuffer.buffer, 0, VK_INDEX_TYPE_UINT32);
        vkCmdBindDescriptorSets(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->geometryPass.layout, 0,
            1, &p->uniformDescriptorSet, 1, unioffsets);        
        vkCmdBindDescriptorSets(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->geometryPass.layout, 1,
            1, &obj[i].textureDescSet, 0, NULL);
        vkCmdDrawIndexed(cmdbuf, obj[i].indexCount, 1, 0, 0, 0);
    }

    vkCmdEndRendering(cmdbuf);
}