#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <vulkan/vulkan_core.h>
#include "../../helpers.h"
#include "../vulkan/renderingpipeline.h"
#include "data.h"

#include "pipeline.h"
#include "setup.h"

void createGeometryPassSD(VkSubpassDescription *desc){
    VkAttachmentReference *refrences = calloc(1, sizeof(VkAttachmentReference));
    VkAttachmentReference colorattachmentref = {
        0,
        VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL
    };
    refrences[0] = colorattachmentref;

    VkSubpassDescription gpdesc = {
        0,
        VK_PIPELINE_BIND_POINT_GRAPHICS,
        0,
        NULL,
        1,
        refrences,
        NULL,
        NULL,
        0,
        NULL
    };
    *desc = gpdesc;
}

void createGeometryPass(struct rendering_state *s, struct pipeline3d *p, uint32_t subpassnum){
    // Shader Code
    sizedData vertshdcode = readFileR("shd/triangle.vert.spv");
    sizedData fragshdcode = readFileR("shd/triangle.frag.spv");

    VkPipelineShaderStageCreateInfo vert = loadshader(vertshdcode, VK_SHADER_STAGE_VERTEX_BIT, s);
    VkPipelineShaderStageCreateInfo frag = loadshader(fragshdcode, VK_SHADER_STAGE_FRAGMENT_BIT, s);

    VkPipelineShaderStageCreateInfo stages[] = {vert, frag};


    // Vertex Input
    VkVertexInputBindingDescription bindes[1];
    bindes[0].binding = 0;
    bindes[0].inputRate = VK_VERTEX_INPUT_RATE_VERTEX;
    bindes[0].stride = sizeof(struct vertex);

    VkVertexInputAttributeDescription atdes[2];
    atdes[0].location = 0;
    atdes[0].binding = 0;
    atdes[0].format = VK_FORMAT_R32G32B32_SFLOAT;
    atdes[0].offset = 0;
    
    atdes[1].location = 1;
    atdes[1].binding = 0;
    atdes[1].format = VK_FORMAT_R32G32_SFLOAT;
    atdes[1].offset = sizeof(float) * 3;


    // Fixed Function States
    VkPipelineVertexInputStateCreateInfo inputState = svlk_createPipelineVertexInputStateCI(bindes, 1, atdes, 2, 0);
    VkPipelineInputAssemblyStateCreateInfo inputAssemblyState = svlk_createPipelineInputAssemblyStateCI(VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST, 0, VK_FALSE);
    VkPipelineViewportStateCreateInfo viewportState = svlk_createPipelineViewportStateCI(1, 1, 0);
    VkPipelineRasterizationStateCreateInfo rasterizerState = svlk_createPipelineRasterizationStateCI(VK_POLYGON_MODE_FILL, VK_CULL_MODE_NONE, VK_FRONT_FACE_CLOCKWISE, 0);
    VkPipelineMultisampleStateCreateInfo multisampleState = svlk_createPipelineMultisampleStateCI(VK_SAMPLE_COUNT_1_BIT, 0);
    VkPipelineDepthStencilStateCreateInfo depthState = svlk_createDepthStencilStateCI(VK_TRUE, VK_FALSE, VK_COMPARE_OP_GREATER_OR_EQUAL);
    VkPipelineColorBlendAttachmentState colorblendatt = svlk_createColorBlendAttachmentState(0xf, VK_FALSE);
    VkPipelineColorBlendStateCreateInfo colorblendState = svlk_createColorBlendStateCI(1, &colorblendatt);
    VkDynamicState dynstates[] = {VK_DYNAMIC_STATE_VIEWPORT, VK_DYNAMIC_STATE_SCISSOR};
    VkPipelineDynamicStateCreateInfo dynamicState = svlk_createDynamicStateCI(dynstates, 2, 0);

    // Pipeline Layout
    VkDescriptorSetLayoutBinding lbu[] = {
        {
            0,
            VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
            1,
            VK_SHADER_STAGE_VERTEX_BIT,
            NULL
        },
    };

    VkDescriptorSetLayoutBinding lbt[] = {
        {
            0,
            VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
            1,
            VK_SHADER_STAGE_FRAGMENT_BIT,
            NULL
        }
    };

    VkDescriptorSetLayoutCreateInfo dslCI[] = {
        {
            VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO,
            NULL,
            0,
            sizeof(lbu)/sizeof(lbu[0]),
            lbu
        },
        {
            VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO,
            NULL,
            0,
            sizeof(lbt)/sizeof(lbt[0]),
            lbt
        }
    };

    vkCreateDescriptorSetLayout(s->device, &dslCI[0], NULL, &p->geometryPass.uniformDesclayout);
    vkCreateDescriptorSetLayout(s->device, &dslCI[1], NULL, &p->geometryPass.textureDesclayout);
    VkDescriptorSetLayout desclayouts[] = {p->geometryPass.uniformDesclayout, p->geometryPass.textureDesclayout};

    VkPipelineLayoutCreateInfo layoutCI = svlk_createPipelineLayoutCI(desclayouts, 2);
    vkCreatePipelineLayout(s->device, &layoutCI, NULL, &p->geometryPass.layout);

    VkPipelineRenderingCreateInfo pipelineRenderingCI = {
        VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO,
        NULL,
        0,
        1,
        &s->surfaceformat.format,
        0,
        0
    };

    VkGraphicsPipelineCreateInfo pipelineCI = {
        VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO,
        &pipelineRenderingCI,
        0,
           2,
        stages,
        &inputState,
        &inputAssemblyState,
        NULL,
        &viewportState,
        &rasterizerState,
        &multisampleState,
        &depthState,
        &colorblendState,
        &dynamicState,
        p->geometryPass.layout,
        NULL,
        subpassnum,
        NULL,
        -1
    };
    
    vkCreateGraphicsPipelines(s->device, NULL, 1, &pipelineCI, NULL, &p->geometryPass.pipeline);
}

VkRenderPassCreateInfo createRenderPass(uint32_t attachmentcount, VkAttachmentDescription *attdes, uint32_t subpasscount, VkSubpassDescription *subpasses){
    VkRenderPassCreateInfo renderpassCI = {
        VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO,
        NULL,
        0,
        attachmentcount,
        attdes,
        subpasscount,
        subpasses,
    };

    return renderpassCI;
}

void createPipeline3d(struct rendering_state *s){
    struct pipeline3d *p = &s->pipeline;

    VkAttachmentDescription attdescs[1] = {0};

    VkAttachmentDescription colordesc = {
        0,
        s->surfaceformat.format,
        VK_SAMPLE_COUNT_1_BIT,
        VK_ATTACHMENT_LOAD_OP_CLEAR,
        VK_ATTACHMENT_STORE_OP_STORE,
        VK_ATTACHMENT_LOAD_OP_DONT_CARE,
        VK_ATTACHMENT_STORE_OP_DONT_CARE,
        VK_IMAGE_LAYOUT_UNDEFINED,
        VK_IMAGE_LAYOUT_PRESENT_SRC_KHR
    }; attdescs[0] = colordesc;

    VkSubpassDescription subpasses[1] = {0};
    createGeometryPassSD(&subpasses[0]);

    VkRenderPassCreateInfo rpCI = createRenderPass(1, attdescs, 1, subpasses);
    vkCreateRenderPass(s->device, &rpCI, NULL, &p->renderpass);

    createGeometryPass(s, p, 0);

    // uniform stuff
    p->uniformHeap.len = 16;
    p->uniformHeap.objsize = sizeof(struct uniformData);
    elementHeap_init(&p->uniformHeap);

    p->uniformbuffer.size = p->uniformHeap.len * p->uniformHeap.objsize;
    rendering_createBuffer(&p->uniformbuffer,
        VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT,
        VK_SHARING_MODE_EXCLUSIVE,
        VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
        s
    );
    vkMapMemory(s->device, p->uniformbuffer.memory, 0, p->uniformbuffer.size, 0, &p->uniformHeap.buffer);

    // why descriptors, why. why museth you exist
    VkDescriptorPoolSize poolsize[] = {
        {
            VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, // offsets are cool
            1 // no linked lists yet so 1
        },
        {
            VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
            1 * p->uniformHeap.len
        }
    };
    
    VkDescriptorPoolCreateInfo dpCI = {
        VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO,
        NULL,
        0,
        poolsize[0].descriptorCount + poolsize[1].descriptorCount*1,
        2,
        poolsize
    };

    VkResult result = vkCreateDescriptorPool(s->device, &dpCI, NULL, &p->descriptorPool);
    if(result != VK_SUCCESS){
        printf("Could not create descriptor pool. result=%i\n", result);
    }

    VkDescriptorSetAllocateInfo allocInfo = {
        VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO,
        NULL,
        p->descriptorPool,
        1,
        &p->geometryPass.uniformDesclayout
    };
    result = vkAllocateDescriptorSets(s->device, &allocInfo, &p->uniformDescriptorSet);
    if(result != VK_SUCCESS){
        printf("Could not create descriptor set. result=%i\n", result);
    }
    updateDescriptorSet(s);
}

void updateDescriptorSet(struct rendering_state *s){
    struct pipeline3d *p = &s->pipeline;
    VkDescriptorBufferInfo bufferInfo = {
        p->uniformbuffer.buffer,
        0,
        sizeof(struct Pipeline3dUniformBufferObject)
    };

    VkWriteDescriptorSet descriptorWrite = {
        VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET,
        NULL,
        p->uniformDescriptorSet,
        0,
        0,
        1,
        VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
        NULL,
        &bufferInfo,
        NULL
    };

    // pain.
    vkUpdateDescriptorSets(s->device, 1, &descriptorWrite, 0, NULL);
}

void createObjTextureDescriptorSet(struct rendering_state *s, struct renderData *data){
    struct pipeline3d *p = &s->pipeline;
    VkDescriptorSetAllocateInfo allocInfo = {
        VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO,
        NULL,
        p->descriptorPool,
        1,
        &p->geometryPass.textureDesclayout
    };
    checkErrorVk("vkAllocateDescriptorSets",
        vkAllocateDescriptorSets(s->device, &allocInfo, &data->textureDescSet), 
        s->die
    );
}

void updateObjTextureDescriptorSet(struct rendering_state *s, struct renderData *data){
    struct pipeline3d *p = &s->pipeline;
    VkDescriptorImageInfo imginfo[] = {
        {
            data->albedo.sampler,
            data->albedo.view,
            VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL
        }
    };

    VkWriteDescriptorSet descriptorWrite = {
        VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET,
        NULL,
        data->textureDescSet,
        0,
        0,
        1,
        VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
        imginfo,
        NULL,
        NULL
    };
    vkUpdateDescriptorSets(s->device, 1, &descriptorWrite, 0, NULL);
}