#include "render.h"
#include "data.h"
#include "unicode/utf8.h"
#include <cglm/mat4.h>
#include <cglm/vec2.h>
#include <stdint.h>
#include <vulkan/vulkan_core.h>

int stop = 0;

void pipelineText_submitTextElement(struct textElement txt, struct Font font, struct PipelineTextUniformBufferObject *p){
    float x, y; x=y=0;
    for(int i = 0; i < txt.length; i++){
        stbtt_packedchar chardat = {0};
        stbtt_aligned_quad quad = {0};
        stbtt_GetPackedQuad(font.chardata, font.w, font.h, txt.text[i], &x, &y, &quad, 0);
        struct PipelineTextUniformBufferObject *o = &p[i];
        glm_vec2_copy(&quad.s0, o->bp0);
        glm_vec2_copy(&quad.s1, o->bp1);

        if(!stop) {printf("aeae:%f", x);stop=1;}

        vec4 vects[4] = {
            {quad.x0, quad.y0, 0.f, 1.f},
            {quad.x1, quad.y0, 0.f, 1.f},
            {quad.x0, quad.y1, 0.f, 1.f},
            {quad.x1, quad.y1, 0.f, 1.f},
        };

        for(int j = 0; j < 4; j++){
            glm_mat4_mulv(txt.matrix, vects[j], vects[j]);
            glm_vec2_copy(vects[j], o->p[j]);
        }
    }
}

// record commands for rendering Text to a command Buffer. If there are multiple Targets, multiple calls are needed
void pipelineText_recordCommandBuffer(VkCommandBuffer cmdbuf, VkRenderPass rp, VkImageView target, VkExtent2D targetextent, VkImageLayout targetlayout, struct rendering_state *s, struct pipelineText *p, struct textElement *obj, uint32_t objcount){
    VkRenderingAttachmentInfo attachments[1] = {
        {
            VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO,
            NULL,
            target,
            targetlayout,
            VK_RESOLVE_MODE_NONE,
            0,
            0,
            VK_ATTACHMENT_LOAD_OP_LOAD,
            VK_ATTACHMENT_STORE_OP_STORE,
            0, 0, 0, 0
        }
    };

    VkRenderingInfo renderinfo = {
        VK_STRUCTURE_TYPE_RENDERING_INFO,
        NULL,
        0,
        {{0,0}, targetextent},
        1,
        0,
        1,
        attachments,
        NULL,
        NULL
    };
    vkCmdBeginRendering(cmdbuf, &renderinfo);

    vkCmdBindPipeline(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->pipeline);

    VkViewport vp = {
        0, 0,
        targetextent.width, targetextent.height,
        0.f, 1.f
    };

    vkCmdSetViewport(cmdbuf, 0, 1, &vp);
    
    VkRect2D sc = {
        {0,0},
        targetextent
    };
    vkCmdSetScissor(cmdbuf, 0, 1, &sc);

    for(uint32_t i = 0; i < objcount; i++){
        //vkCmdBindDescriptorSets(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->layout, 0, 1, &p->uniformDescriptorSet, 0, NULL);
        //vkCmdBindDescriptorSets(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->layout, 0, 1, &obj[i].font->descriptor, 0, NULL);
        
        
        vkCmdBindDescriptorSets(cmdbuf, VK_PIPELINE_BIND_POINT_GRAPHICS, p->layout, 0, 1, &obj[i].font->descriptor, 0, NULL);
        
        for(int j = 0; j < obj[i].length; j++){
            vkCmdPushConstants(cmdbuf, p->layout, VK_SHADER_STAGE_VERTEX_BIT, 0, sizeof(obj->compiled[0]), &obj[i].compiled[j]);
            vkCmdDraw(cmdbuf, 6, 1, 0, 0);
        }
    }

    vkCmdEndRendering(cmdbuf);
}