#include "setup.h"
#include "../../helpers.h"
#include "../vulkan/renderingpipeline.h"
#include "data.h"
#include "pipeline.h"
#include <vulkan/vulkan_core.h>

void setupMainTextPipeline(struct rendering_state *s, struct pipelineText *p){
    
}

void createPipelineText(struct rendering_state *s){
    struct pipelineText *p = &s->pipelinetext;

    // Shader Code
    sizedData vertshdcode = readFileR("shd/text.vert.spv");
    sizedData fragshdcode = readFileR("shd/text.frag.spv");

    VkPipelineShaderStageCreateInfo vert = loadshader(vertshdcode, VK_SHADER_STAGE_VERTEX_BIT, s);
    VkPipelineShaderStageCreateInfo frag = loadshader(fragshdcode, VK_SHADER_STAGE_FRAGMENT_BIT, s);

    VkPipelineShaderStageCreateInfo stages[] = {vert, frag};



    VkAttachmentDescription attdescs[1] = {0};
    VkAttachmentDescription colordescs = {
        0,
        s->surfaceformat.format,
        VK_SAMPLE_COUNT_1_BIT,
        VK_ATTACHMENT_LOAD_OP_CLEAR,
        VK_ATTACHMENT_STORE_OP_STORE,
        VK_ATTACHMENT_LOAD_OP_DONT_CARE,
        VK_ATTACHMENT_STORE_OP_DONT_CARE,
        VK_IMAGE_LAYOUT_UNDEFINED,
        VK_IMAGE_LAYOUT_GENERAL
    }; attdescs[0] = colordescs;

    VkPipelineVertexInputStateCreateInfo inputState = svlk_createPipelineVertexInputStateCI(NULL, 0, NULL, 0, 0);
    VkPipelineInputAssemblyStateCreateInfo inputAssemblyState = svlk_createPipelineInputAssemblyStateCI(VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST, 0, VK_FALSE);
    VkPipelineViewportStateCreateInfo viewportState = svlk_createPipelineViewportStateCI(1, 1, 0);
    VkPipelineRasterizationStateCreateInfo rasterizerState = svlk_createPipelineRasterizationStateCI(VK_POLYGON_MODE_FILL, VK_CULL_MODE_NONE, VK_FRONT_FACE_CLOCKWISE, 0);
    VkPipelineMultisampleStateCreateInfo multisampleState = svlk_createPipelineMultisampleStateCI(VK_SAMPLE_COUNT_1_BIT, 0);
    VkPipelineDepthStencilStateCreateInfo depthState = svlk_createDepthStencilStateCI(VK_FALSE, VK_FALSE, VK_COMPARE_OP_GREATER_OR_EQUAL);
    VkPipelineColorBlendAttachmentState colorblendatt = svlk_createColorBlendAttachmentState(0xf, VK_TRUE);
    VkPipelineColorBlendStateCreateInfo colorblendState = svlk_createColorBlendStateCI(1, &colorblendatt);
    VkDynamicState dynstates[] = {VK_DYNAMIC_STATE_VIEWPORT, VK_DYNAMIC_STATE_SCISSOR};
    VkPipelineDynamicStateCreateInfo dynamicState = svlk_createDynamicStateCI(dynstates, 2, 0);

    // Pipeline Layout
    VkDescriptorSetLayoutBinding lbu[] = {
        // charpositions
        /*{
            0,
            VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER,
            1,
            VK_SHADER_STAGE_VERTEX_BIT | VK_SHADER_STAGE_FRAGMENT_BIT,
            NULL
        },*/
    };

    VkDescriptorSetLayoutBinding lbt[] = {
        // char atlas
        {
            0,
            VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
            1,
            VK_SHADER_STAGE_FRAGMENT_BIT,
            NULL
        }
    };
    
    VkDescriptorSetLayoutCreateInfo dslCI[] = {
        /*{
            VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO,
            NULL,
            0,
            sizeof(lbu)/sizeof(lbu[0]),
            lbu
        },*/
        {
            VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO,
            NULL,
            0,
            sizeof(lbt)/sizeof(lbt[0]),
            lbt
        }
    };

    //vkCreateDescriptorSetLayout(s->device, &dslCI[0], NULL, &p->uniformDesclayout);
    vkCreateDescriptorSetLayout(s->device, &dslCI[0], NULL, &p->textureDesclayout);
    VkDescriptorSetLayout desclayouts[] = {p->textureDesclayout};

    VkPushConstantRange pcRange = {
        VK_SHADER_STAGE_VERTEX_BIT,
        0,
        sizeof(struct PipelineTextUniformBufferObject)
    };

    VkPipelineLayoutCreateInfo layoutCI = svlk_createPipelineLayoutCI(desclayouts, 1);
    layoutCI.pushConstantRangeCount = 1;
    layoutCI.pPushConstantRanges = &pcRange;
    vkCreatePipelineLayout(s->device, &layoutCI, NULL, &p->layout);

    VkPipelineRenderingCreateInfo pipelineRenderingCI = {
        VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO,
        NULL,
        0,
        1,
        &s->surfaceformat.format,
        0,
        0
    };

    VkGraphicsPipelineCreateInfo pipelineCI = {
        VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO,
        &pipelineRenderingCI,
        0,
        2,
        stages,
        &inputState,
        &inputAssemblyState,
        NULL,
        &viewportState,
        &rasterizerState,
        &multisampleState,
        &depthState,
        &colorblendState,
        &dynamicState,
        p->layout,
        NULL,
        0,
        NULL,
        -1
    };
    vkCreateGraphicsPipelines(s->device, NULL, 1, &pipelineCI, NULL, &p->pipeline);

    VkDescriptorPoolSize poolsize[] = {
        {
            VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
            5
        }
    };

    VkDescriptorPoolCreateInfo dpCI = {
        VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO,
        NULL,
        0,
        5,
        1,
        poolsize
    };
    checkErrorVk("vkCreateDescriptorPool (pipelineText)", 
        vkCreateDescriptorPool(s->device, &dpCI, NULL, &p->descriptorPool), 
        s->die
    );
}


void createFontTextureDesctiptorSet(struct rendering_state *s, struct Font *f){
    struct pipelineText *p = &s->pipelinetext;
    VkDescriptorSetAllocateInfo allocInfo = {
        VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO,
        NULL,
        p->descriptorPool,
        1,
        &p->textureDesclayout
    };
    checkErrorVk("vkAllocateDescriptorSets (pipelinetext)",
        vkAllocateDescriptorSets(s->device, &allocInfo, &f->descriptor), 
        s->die
    );
}

void updateFontTextureDescriptorSet(struct rendering_state *s, struct Font *f){
    struct pipelineText *p = &s->pipelinetext;
    VkDescriptorImageInfo imginfo[] = {
        {
            f->buffer->sampler,
            f->buffer->view,
            VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL
        }
    };

    VkWriteDescriptorSet descriptorWrite = {
        VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET,
        NULL,
        f->descriptor,
        0,
        0,
        1,
        VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
        imginfo,
        NULL,
        NULL
    };
    vkUpdateDescriptorSets(s->device, 1, &descriptorWrite, 0, NULL);
}