#ifndef __RENDERING_STRUCT_H__
#define __RENDERING_STRUCT_H__

#include "camera.h"
#include "pipeline3d/pipeline.h"
#include "pipelineText/pipeline.h"
#include "vulkan/gpumem_types.h"
#include <GLFW/glfw3.h>
#include <stdint.h>
#include <vulkan/vulkan_core.h>

struct rendering_queueFamilies{
    int graphicsQFI;
    VkQueue graphicsQueue;
    VkQueue transferQueue; // for staging buffers and such
    int presentationQFI;
    VkQueue presentationQueue;
    
    // if this struct is initialised yet
    int present;
};

//general rendering settings (eg multisampling etc.)
struct rendering_settings{

};

// semaphores and fences
struct rendering_sync{
    VkFence frameFinishFence;
    VkSemaphore presentReadySemaphore;
    VkSemaphore renderFinishSemaphore;
};

/* moved to rendering/pipeline3d/vertex.h
struct vertex{
    // position
    float x;
    float y;
    float z;
    // texture
    float u;
    float v;
};*/

struct rendering_state{
    // must be filled out prior to initialising glfw
    int window_width;
    int window_height;
    char *window_title;

    // the last error that occured
    VkResult error;

    GLFWwindow *window;

    const char **extensions;
    uint32_t numextensions;

    const char **deviceextensions;
    uint32_t numdeviceextensions;
    
    const char **layers;
    uint32_t numlayers;

    VkInstance instance;
    VkSurfaceKHR surface;
    VkPhysicalDevice physdevice;
    struct rendering_queueFamilies queues;
    VkDevice device;
    
    VkSurfaceFormatKHR surfaceformat;
    VkSurfaceCapabilitiesKHR surfaceCapabilities;
    VkExtent2D swapchainExtent;
    VkSwapchainKHR swapchain;
    int swapchainNumimgs;
    VkImage *swapchainImages;
    uint32_t numswapchainImages;
    VkImageView *swapchainImageView;
    uint32_t numswapchainImageView;
    VkFramebuffer *framebuffers;
    uint32_t numframebuffers;

    VkGraphicsPipelineCreateInfo *gptemplate;

    struct pipeline3d pipeline;
    struct pipelineText pipelinetext;

    VkCommandPool graphicsPool;
    VkCommandBuffer graphicsBuffer;
    VkCommandBuffer transferBuffer;

    struct gpuimage renderTarget;

    struct rendering_sync sync;

    uint32_t shouldRecreateswapchainFlags;
    // filled in when framebuffer size changes
    int tmp_window_width;
    int tmp_window_height;

    void (*die)();
};

enum rendering_SwapchainRecreateFlags{
    RNDR_RECREATE_SWAPCHAIN = 1,
    RNDR_RECREATE_SWAPCHAIN_USE_TMP = 2 // use tmp_window_width & height
};

struct rendering_windowUserPointer{
    struct camera *maincam;
    struct rendering_state *s;
};

#endif