#ifndef __GPUMEM_H__
#define __GPUMEM_H__
#include "../struct.h"
#include "gpumem_types.h"
#include <vulkan/vulkan_core.h>

void rendering_createBuffer(struct gpubuffer *buf, VkBufferUsageFlagBits usage, VkSharingMode shareMode, VkMemoryPropertyFlagBits properties, struct rendering_state *s);

uint32_t rendering_findMemoryType(uint32_t type, VkMemoryPropertyFlags properties, struct rendering_state *s);

void rendering_copyBuffer(VkBuffer src, VkBuffer dst, VkDeviceSize size, struct rendering_state *s);

void rendering_putBuffer(struct gpubuffer *buf, void *data, struct rendering_state *s);

void rendering_makeBufferForRender(struct gpubuffer *buf, void *data, VkBufferUsageFlagBits usage, VkSharingMode shareMode, struct rendering_state *s);

static inline void rendering_destroyBuffer(struct gpubuffer *b, struct rendering_state *s){
    vkDestroyBuffer(s->device, b->buffer, NULL);
    vkFreeMemory(s->device, b->memory, NULL);
}

void rendering_createImage(
    uint32_t w, uint32_t h, uint32_t d, VkImageType type, VkFormat fmt, uint32_t mipLevels, VkImageTiling tiling,
    VkImageUsageFlags usageflags, VkMemoryPropertyFlags properties, struct gpuimage *img, struct rendering_state *s);

void rendering_copyImageB(VkBuffer src, VkImage img, uint32_t w, uint32_t h, uint32_t d, VkImageAspectFlags aspect, struct rendering_state *s);

void rendering_transitionImageLayoutCB(VkCommandBuffer buf, VkImage img, VkFormat fmt, VkImageLayout old, VkImageLayout newlayout);
void rendering_transitionImageLayout(VkImage img, VkFormat fmt, VkImageLayout old, VkImageLayout newlayout, struct rendering_state *s);

void rendering_createImageForRender(uint32_t w, uint32_t h, void *data, uint64_t datalen, VkFormat fmt, struct gpuimage *img, struct rendering_state *s);
void rendering_destroyImage(struct gpuimage *img, struct rendering_state *s);
//void rendering_createImage();

#endif