#include "svulc.h"
#include <stdlib.h>
#include <vulkan/vulkan_core.h>
#include <unistd.h>

VkResult svlk_createIinstance(const char **layers, int layersNum, const char **extensions, int extensionsNum, VkInstance *instance){
    VkInstanceCreateInfo createInfo = {0};
    createInfo.sType = VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO;
    createInfo.pNext = NULL;
    createInfo.flags = 0;

    VkApplicationInfo appinfo = {
        VK_STRUCTURE_TYPE_APPLICATION_INFO,
        NULL,
        NULL,
        0,
        NULL,
        0,
        VK_API_VERSION_1_4
    };

    createInfo.pApplicationInfo = &appinfo;
    createInfo.enabledLayerCount = layersNum;
    createInfo.ppEnabledLayerNames = layers;
    createInfo.enabledExtensionCount = extensionsNum;
    createInfo.ppEnabledExtensionNames = extensions;

    return vkCreateInstance(&createInfo, NULL, instance);
}

VkResult svlk_getPhysDeviceList(VkInstance instace, unsigned int *num, VkPhysicalDevice **devicesPointer){
    VkResult result = vkEnumeratePhysicalDevices(instace, num, NULL);
    if(result != VK_SUCCESS){
        return result;
    }
    *devicesPointer = malloc(*num * sizeof(VkPhysicalDevice));
    result = vkEnumeratePhysicalDevices(instace, num, *devicesPointer);
    if(result != VK_SUCCESS){
        return result;
    }
    return result;
}

int svlk_getQueueFamilyIndex(VkPhysicalDevice physDevice, VkQueueFlags queueFlag, int *error){
    unsigned int len = 0;
    vkGetPhysicalDeviceQueueFamilyProperties(physDevice, &len, NULL);
    VkQueueFamilyProperties queueFamilies[len];
    vkGetPhysicalDeviceQueueFamilyProperties(physDevice, &len, queueFamilies);
    if(error) *error=0;

    for (int i=0; i<len; i++) {
        if(queueFamilies[i].queueFlags & queueFlag){
            return i;
        }
    }
    if(error) *error=1;
    return 0;
}

int svlk_getPresentQueueFamilyIndex(VkPhysicalDevice physDevice, VkSurfaceKHR surface, int *error){
    unsigned int len = 0;
    vkGetPhysicalDeviceQueueFamilyProperties(physDevice, &len, NULL);
    VkQueueFamilyProperties queueFamilies[len];
    vkGetPhysicalDeviceQueueFamilyProperties(physDevice, &len, queueFamilies);
    if(error) *error=0;    

    for (int i=0; i<len; i++) {
        VkBool32 supportsSurface=0;
        vkGetPhysicalDeviceSurfaceSupportKHR(physDevice, i, surface, &supportsSurface);
        if(supportsSurface){
            return i;
        }
    }
    if(error) *error=1;
    return 0;
}

int svlk_deviceCompatable(VkPhysicalDevice physDevice, VkSurfaceKHR surface, unsigned int queueFamilies, const char **deviceExtensions, int deviceExtensionsNum){
    int ret = 1;
    for (unsigned int mask = 0x80000000; mask != 0; mask >>= 1) {
        if(queueFamilies & mask){
            int error=0;
            svlk_getQueueFamilyIndex(physDevice, queueFamilies & mask, &error);
            if(error) ret=0;
        }
    }
    // not all queue families were found
    if (!ret) return 0;

    if (surface) {
        // check if rendering to the surface is possible
        unsigned int num = 0;
        vkGetPhysicalDeviceSurfaceFormatsKHR(physDevice, surface, &num, NULL);
        if(!num) return 0;
        vkGetPhysicalDeviceSurfacePresentModesKHR(physDevice, surface, &num, NULL);
        if(!num) return 0;

        // check for present family
        int error = 0;
        svlk_getPresentQueueFamilyIndex(physDevice, surface, &error);
        if(!error) return 1;
        else return 0;
    }

    return 1;
}

VkDeviceQueueCreateInfo svlk_createQueueInfo(int queueIndex, int count, float *priority){
    VkDeviceQueueCreateInfo ret={0};
    ret.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO;
    ret.pNext = NULL;
    ret.flags = 0;
    ret.queueFamilyIndex = queueIndex;
    ret.queueCount = 1;
    ret.pQueuePriorities = priority;
    return ret;
}

VkResult svlk_createLogicalDevice(
    VkPhysicalDevice physDevice,
    VkSurfaceKHR surface,
    VkDeviceQueueCreateInfo *queueFamilies,
    unsigned int queueFamiliesNum,
    VkPhysicalDeviceFeatures deviceFeatures,
    const char **deviceExtensions,
    int deviceExtensionsNum,
    void *pNext,
    VkDevice *device
    ){

    VkDeviceCreateInfo createInfo = {0};
    createInfo.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
    createInfo.pNext = pNext;
    createInfo.flags = 0;
    createInfo.queueCreateInfoCount = queueFamiliesNum;
    createInfo.pQueueCreateInfos = queueFamilies;
    createInfo.enabledLayerCount = 0;
    createInfo.enabledExtensionCount = deviceExtensionsNum;
    createInfo.ppEnabledExtensionNames = deviceExtensions;
    createInfo.pEnabledFeatures = &deviceFeatures;

    return vkCreateDevice(physDevice, &createInfo, NULL, device);
}