#ifndef __RENDERING_PIPELINE_H__
#define __RENDERING_PIPELINE_H__
#include <vulkan/vulkan_core.h>
#include "../struct.h"
#include "../../helpers.h"
#include "../init.h"

static inline VkPipelineShaderStageCreateInfo loadshader(sizedData code, VkShaderStageFlagBits flags, struct rendering_state *s){
    VkShaderModule mod = rendering_createShaderModule(code, s->device);

    VkPipelineShaderStageCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
        NULL,
        0,
        flags,
        mod,
        "main",
        NULL
    };
    return CI;
};

static inline VkPipelineVertexInputStateCreateInfo svlk_createPipelineVertexInputStateCI(VkVertexInputBindingDescription *bindes, uint32_t bindescount, VkVertexInputAttributeDescription *atdes, uint32_t atdescount, VkPipelineVertexInputStateCreateFlags flags){
    VkPipelineVertexInputStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO,
        NULL,
        flags,
        bindescount,
        bindes,
        atdescount,
        atdes
    };
    return CI;
}
static inline VkPipelineInputAssemblyStateCreateInfo svlk_createPipelineInputAssemblyStateCI(VkPrimitiveTopology ptop, VkPipelineInputAssemblyStateCreateFlags flags, VkBool32 primitiveRestart){
    VkPipelineInputAssemblyStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO,
        NULL,
        flags,
        ptop,
        primitiveRestart
    };
    return CI;
}
static inline VkPipelineViewportStateCreateInfo svlk_createPipelineViewportStateCI(uint32_t viewportCount, uint32_t scissorCount, VkPipelineViewportStateCreateFlags flags){
    VkPipelineViewportStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO,
        NULL,
        flags,
        viewportCount,
        NULL,
        scissorCount,
        NULL
    };
    return CI;
}
static inline VkPipelineRasterizationStateCreateInfo svlk_createPipelineRasterizationStateCI(VkPolygonMode pmode, VkCullModeFlags cmf, VkFrontFace ff, VkPipelineRasterizationStateCreateFlags flags){
    VkPipelineRasterizationStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO,
        NULL,
        flags,
        0,
        0,
        pmode,
        cmf,
        ff,
        0,
        0.f,
        0.f,
        0.f,
        1.f
    };
    return CI;
}
static inline VkPipelineMultisampleStateCreateInfo svlk_createPipelineMultisampleStateCI(VkSampleCountFlagBits scf, VkPipelineMultisampleStateCreateFlags flags){
    VkPipelineMultisampleStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO,
        NULL,
        flags,
        scf,
        0,
        0.f,
        NULL,
        0,
        0
    };
    return CI;
}
static inline VkPipelineDepthStencilStateCreateInfo svlk_createDepthStencilStateCI(VkBool32 depthTest, VkBool32 depthWrite, VkCompareOp depthop){
    VkPipelineDepthStencilStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO,
        NULL,
        0,
        depthTest,
        depthWrite,
        depthop,
        0,
        0,
        {},
        {.compareOp=VK_COMPARE_OP_ALWAYS},
        0,
        0
    };
    return CI;
}
static inline VkPipelineColorBlendAttachmentState svlk_createColorBlendAttachmentState(VkColorComponentFlags colorWriteMask, VkBool32 blend){
    // vulkan dev, aka full time struct filler
    VkPipelineColorBlendAttachmentState state = {
        blend,
        VK_BLEND_FACTOR_SRC_ALPHA,
        VK_BLEND_FACTOR_ONE,
        VK_BLEND_OP_ADD,
        VK_BLEND_FACTOR_ONE,
        VK_BLEND_FACTOR_ZERO,
        VK_BLEND_OP_ADD,
        colorWriteMask
    };
    return state;
}
static inline VkPipelineColorBlendStateCreateInfo svlk_createColorBlendStateCI(uint32_t attcount, VkPipelineColorBlendAttachmentState *atts){
    VkPipelineColorBlendStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO,
        NULL,
        0,
        0,
        VK_LOGIC_OP_OR,
        attcount,
        atts,
        0,
        0,
        0,
        0
    };
    return CI;
}
static inline VkPipelineDynamicStateCreateInfo svlk_createDynamicStateCI(VkDynamicState *states, uint32_t count, VkPipelineDynamicStateCreateFlags flags){
    VkPipelineDynamicStateCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO,
        NULL,
        flags,
        count,
        states
    };
    return CI;
}
/*inline VkRenderPassCreateInfo svlk_createRenderPassCI(VkAttachmentDescription *att, uint32_t attcount, VkSubpassDescription *sub, uint32_t supcount, VkSubpassDependency *sud, uint32_t sudcount, VkRenderPassCreateFlags flags){

}*/

static inline VkPipelineLayoutCreateInfo svlk_createPipelineLayoutCI(VkDescriptorSetLayout *layouts, uint32_t count){
    VkPipelineLayoutCreateInfo CI = {
        VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO,
        NULL,
        0,
        count,
        layouts,
        0,
        NULL
    };
    return CI;
}




/*
const VkPipelineShaderStageCreateInfo svlk_vertexShaderTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
    NULL,
    0,
    VK_SHADER_STAGE_VERTEX_BIT,
    NULL, // must be changed by user
    "main",
    NULL
};

const VkPipelineShaderStageCreateInfo svlk_fragmentShaderTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
    NULL,
    0,
    VK_SHADER_STAGE_FRAGMENT_BIT,
    NULL, // must be changed by user
    "main",
    NULL
};

const VkPipelineMultisampleStateCreateInfo svlk_noMultisample = {
    VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO,
    NULL,
    0,
    VK_SAMPLE_COUNT_1_BIT,
    VK_FALSE,
    1.0f,
    NULL,
    VK_FALSE,
    VK_FALSE
};

const VkPipelineVertexInputStateCreateInfo svlk_noVertexData = {
    VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO,
    NULL,
    0,
    0,
    NULL,
    0,
    NULL
};

const VkPipelineInputAssemblyStateCreateInfo svlk_inputAssemblyTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO,
    NULL,
    0,
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST,
    VK_FALSE
};

const VkViewport svlk_viewportTemplate = {
    0.0f,
    0.0f,
    0.0f,    // width and height must be set by user
    0.0f,   // width and height must be set by user
    0.0f,
    1.0f
};

const VkPipelineRasterizationStateCreateInfo svlk_rasterizationTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO,
    NULL,
    0,
    VK_FALSE,
    VK_FALSE,
    VK_POLYGON_MODE_FILL,
    VK_CULL_MODE_BACK_BIT,
    VK_FRONT_FACE_CLOCKWISE,
    VK_FALSE,
    0.0f,
    0.0f,
    0.0f,
    1.0f
};

const VkPipelineColorBlendAttachmentState svlk_colorBlendTemplate = {
    VK_FALSE,

    VK_BLEND_FACTOR_ONE,
    VK_BLEND_FACTOR_ZERO,
    VK_BLEND_OP_ADD,

    VK_BLEND_FACTOR_ONE,
    VK_BLEND_FACTOR_ZERO,
    VK_BLEND_OP_ADD,
    VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT
};

const VkPipelineColorBlendStateCreateInfo svlk_colorBlendCreateInfoTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO,
    NULL,
    0,
    VK_FALSE,
    VK_LOGIC_OP_COPY,
    0, // must be set by user
    NULL, // must be set by user
    0.f,
    0.f,
    0.f,
    0.f
};

const VkPipelineLayoutCreateInfo svlk_pipelineLayoutCreateInfoTemplate = {
    VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO,
    NULL,
    0,
    0,
    NULL,
    0,
    NULL
};
*/
#endif